#!/usr/bin/perl

## no critic (RequireCheckedSyscalls);

use strict;
use warnings;
use autodie;

use Carp;
use IPC::Run;
use Sys::Hostname;

my @NAGIOS_SERVER_IPS = ('171.67.217.115',);

my $DEBUG = 0;

my $check_script = '/usr/sbin/check-repo-inrelease-valid';

# Use: pass in an array, returns ($stdout, $stderr, $exit_value)
sub run_command {
    my (@command) = @_;

    # (From Russ)
    my ($out, $err);
    IPC::Run::run(\@command, q{>}, \$out, q{2>}, \$err);
    return ($out, $err, $? >> 8);
}

sub progress {
    my ($msg) = @_;
    if ($DEBUG) {
        print $msg . "\n";
    }
    return;
}

# Process any options:
use Getopt::Long::Descriptive;

my ($opt, $usage) = describe_options(
    'monitor-distro-inrelease-valid-nagios %o',
    ['repository|r=s', 'query this repository',      { required => 1 }],
    ['security|s',     'this is a security archive', { default  => 0 }],
    ['verbose|v',      'print extra stuff'],
    ['help',           'print usage message and exit'],
);

if ($opt->verbose()) {
    $DEBUG = 1;
}

my $repository;
if ($opt->repository()) {
    $repository = $opt->repository();
} else {
    my $hostname = hostname();
    $repository = "http://${hostname}.stanford.edu/debian";
}

my $security;
if ($opt->security()) {
    $security = 1;
} else {
    $security = 0;
}

my $hostname;
if ($repository =~ m{\/\/([^.]+[.][\w.]*)}xsm) {
    $hostname = $1;
} else {
    croak("could not find hostname in '$repository'");
}

# Remove any trailing slash.
$repository =~ s{\/$}{}xsm;

progress("using repository '$repository'");

# Mirroring is considered critically behind if the InRelease file's
# expiration is less than $CRIT_SECS seconds from now, and is considered
# worringly behind if its expiration is less than $WARN_SECS seconds from
# now.
my $WARN_SECS = 60 * 60 * 24 * 4;    # Four days.
my $CRIT_SECS = 60 * 60 * 24 * 2;    # Two days.
progress("WARN     seconds is $WARN_SECS");
progress("CRITICAL seconds is $CRIT_SECS");

sub process {
    my ($distr_array_aref) = @_;

    my @distro_array = @{$distr_array_aref};

    foreach my $distro_var (@distro_array) {

        # Get the number of seconds since last update.
        my @cmd = ($check_script, $repository, $distro_var,);

        if ($security) {
            push(@cmd, 'security');
        }

        my ($stdout, $stderr, $rc) = run_command(@cmd);
        my $seconds_behind = $stdout;

        progress("seconds_behind is $seconds_behind");

        my $msg;
        ## no critic (ProhibitCascadingIfElse);
        if ($stderr) {
            $msg = "3;UNKNOWN - monitor failed: $stderr";
        } elsif ($seconds_behind eq q{}) {
            $msg
              = '3;UNKNOWN - check_repo_inrelease_valid returned an empty string';
        } elsif ($seconds_behind < $CRIT_SECS) {
            $msg
              = "2;CRITICAL - $distro_var is behind $seconds_behind seconds";
        } elsif ($seconds_behind < $WARN_SECS) {
            $msg = "1;WARNING - $distro_var is behind $seconds_behind seconds";
        } else {
            $msg = '0;OK - Valid_Until is within system limits';
        }

        # Send NCSA (passive check) to Nagios monitors.
        my $test_name;
        if ($security) {
            $test_name = "m-security-${distro_var}";
        } else {
            $test_name = "m-${distro_var}";
        }

        foreach my $nagios_server_ip (@NAGIOS_SERVER_IPS) {
            my $psv_msg = "$hostname;$test_name;$msg";
            progress("passive message: $psv_msg");

            my $out;
            #<<<  perltidy please ignore this
            my @command = (
                'send_nsca',
                '-d', q{;},
                '-H', $nagios_server_ip,
                );
            #>>>
            IPC::Run::run \@command, \$psv_msg, \$out or croak $?;
            progress("send_ncsa command output: $out");
        }
    }

    return;
}

# We look at these distributions.
my @distro_array_ordinary = qw(
  stable-proposed-updates
  stable-updates
  stable-backports
  oldstable-proposed-updates
  oldstable-updates
  oldstable-backports
  testing
  testing-proposed-updates
  testing-updates
  sid
);

# We look at these debian-security distributions.
my @debian_security_distro_array = qw(
  stable
  oldstable
  oldoldstable
);

if ($security) {
    process(\@debian_security_distro_array);
} else {
    process(\@distro_array_ordinary);
}

__END__

=head1 NAME

monitor-distro-inrelease-valid-nagios.pl - Debian repo passive monitoring

=head1 SYNOPSIS

B<monitor-distro-inrelease-valid-nagios.pl> debian-mirror-endpoint

=head1 DESCRIPTION

Downloads the InRelease file for several Debian distributions at the
Debian repository pointed to by C<debian-mirror-endpoint>. It iterates
over the following distributions looking for out-of-date InRelease files:

    sid
    buster
    buster-updates
    buster-proposed-updates
    stretch-updates
    stretch-proposed-updates
    stretch-backports
    jessie-backports
    jessie-updates
    jessie-proposed-updates

=head1 EXAMPLES

    monitor-distro-inrelease-valid-nagios.pl http://debian.stanford.edu/debian
    monitor-distro-inrelease-valid-nagios.pl http://debian.stanford.edu/debian-security

=head1 SEE ALSO

debmirror(1)

=head1 AUTHORS

Srinivas Puttagunta <psr123@stanford.edu>.

=cut
